@@ -70,6 +70,7 @@ ID	oj_instance_variables_id;
 ID	oj_json_create_id;
 ID	oj_length_id;
 ID	oj_new_id;
+ID	oj_parse_id;
 ID	oj_pos_id;
 ID	oj_read_id;
 ID	oj_readpartial_id;
@@ -84,7 +85,9 @@ ID	oj_to_time_id;
 ID	oj_tv_nsec_id;
 ID	oj_tv_sec_id;
 ID	oj_tv_usec_id;
+ID	oj_utc_id;
 ID	oj_utc_offset_id;
+ID	oj_utcq_id;
 ID	oj_write_id;
 
 VALUE	oj_bag_class;
@@ -97,7 +100,6 @@ VALUE	oj_stream_writer_class;
 VALUE	oj_string_writer_class;
 VALUE	oj_stringio_class;
 VALUE	oj_struct_class;
-VALUE	oj_time_class;
 
 VALUE	oj_slash_string;
 
@@ -130,6 +132,7 @@ static VALUE	strict_sym;
 static VALUE	symbol_keys_sym;
 static VALUE	time_format_sym;
 static VALUE	unix_sym;
+static VALUE	unix_zone_sym;
 static VALUE	use_to_json_sym;
 static VALUE	xmlschema_sym;
 static VALUE	xss_safe_sym;
@@ -164,7 +167,7 @@ struct _Options	oj_default_options = {
     JSONEsc,		// escape_mode
     ObjectMode,		// mode
     Yes,		// class_cache
-    UnixTime,		// time_format
+    UnixZTime,		// time_format
     Yes,		// bigdec_as_num
     AutoDec,		// bigdec_load
     Yes,		// to_json
@@ -191,7 +194,7 @@ static VALUE	define_mimic_json(int argc, VALUE *argv, VALUE self);
  * - escape_mode: [:newline|:json|:xss_safe|:ascii|nil] determines the characters to escape
  * - class_cache: [true|false|nil] cache classes for faster parsing (if dynamically modifying classes or reloading classes then don't use this)
  * - mode: [:object|:strict|:compat|:null] load and dump modes to use for JSON
- * - time_format: [:unix|:xmlschema|:ruby] time format when dumping in :compat mode
+ * - time_format: [:unix|:unix_zone|:xmlschema|:ruby] time format when dumping in :compat and :object mode
  * - bigdecimal_as_decimal: [true|false|nil] dump BigDecimal as a decimal number or as a String
  * - bigdecimal_load: [:bigdecimal|:float|:auto] load decimals as BigDecimal instead of as a Float. :auto pick the most precise for the number of digits.
  * - create_id: [String|nil] create id for json compatible object encoding, default is 'json_create'
@@ -236,6 +239,7 @@ get_def_opts(VALUE self) {
     switch (oj_default_options.time_format) {
     case XmlTime:	rb_hash_aset(opts, time_format_sym, xmlschema_sym);	break;
     case RubyTime:	rb_hash_aset(opts, time_format_sym, ruby_sym);		break;
+    case UnixZTime:	rb_hash_aset(opts, time_format_sym, unix_zone_sym);	break;
     case UnixTime:
     default:		rb_hash_aset(opts, time_format_sym, unix_sym);		break;
     }
@@ -275,6 +279,7 @@ get_def_opts(VALUE self) {
  *	  replaces them with a null.
  * @param [:unix|:xmlschema|:ruby] time format when dumping in :compat mode
  *        :unix decimal number denoting the number of seconds since 1/1/1970,
+ *        :unix_zone decimal number denoting the number of seconds since 1/1/1970 plus the utc_offset in the exponent ,
  *        :xmlschema date-time format taken from XML Schema as a String,
  *        :ruby Time.to_s formatted String
  * @param [String|nil] :create_id create id for json compatible object encoding
@@ -360,12 +365,14 @@ set_def_opts(VALUE self, VALUE opts) {
 	// ignore
     } else if (unix_sym == v) {
 	oj_default_options.time_format = UnixTime;
+    } else if (unix_zone_sym == v) {
+	oj_default_options.time_format = UnixZTime;
     } else if (xmlschema_sym == v) {
 	oj_default_options.time_format = XmlTime;
     } else if (ruby_sym == v) {
 	oj_default_options.time_format = RubyTime;
     } else {
-	rb_raise(rb_eArgError, ":time_format must be :unix, :xmlschema, or :ruby.");
+	rb_raise(rb_eArgError, ":time_format must be :unix, :unix_zone, :xmlschema, or :ruby.");
     }
 
     v = rb_hash_lookup(opts, escape_mode_sym);
@@ -512,12 +519,14 @@ oj_parse_options(VALUE ropts, Options copts) {
 	if (Qnil != (v = rb_hash_lookup(ropts, time_format_sym))) {
 	    if (unix_sym == v) {
 		copts->time_format = UnixTime;
+	    } else if (unix_zone_sym == v) {
+		copts->time_format = UnixZTime;
 	    } else if (xmlschema_sym == v) {
 		copts->time_format = XmlTime;
 	    } else if (ruby_sym == v) {
 		copts->time_format = RubyTime;
 	    } else {
-		rb_raise(rb_eArgError, ":time_format must be :unix, :xmlschema, or :ruby.");
+		rb_raise(rb_eArgError, ":time_format must be :unix, :unix_zone, :xmlschema, or :ruby.");
 	    }
 	}
 
@@ -608,6 +617,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
  *
+ * A block can also be provided with a single argument. That argument will be
+ * the parsed JSON document. This is useful when parsing a string that includes
+ * multiple JSON documents.
+ *
  * @param [String|IO] json JSON String or an Object that responds to read()
  * @param [Hash] options load options (same as default_options)
  */
@@ -629,6 +642,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
  *
+ * A block can also be provided with a single argument. That argument will be
+ * the parsed JSON document. This is useful when parsing a string that includes
+ * multiple JSON documents.
+ *
  * @param [String|IO] json JSON String or an Object that responds to read()
  * @param [Hash] options load options (same as default_options)
  */
@@ -666,6 +683,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * different initializer can not be automatically decoded. A way around this is
  * to use a create function but this example shows an alternative.
  *
+ * A block can also be provided with a single argument. That argument will be
+ * the parsed JSON document. This is useful when parsing a string that includes
+ * multiple JSON documents.
+ *
  * @param [String|IO] json JSON String or an Object that responds to read()
  * @param [Hash] options load options (same as default_options)
  */
@@ -686,6 +707,10 @@ oj_parse_options(VALUE ropts, Options copts) {
  * a file object is passed as the first argument. A stream input will be parsed
  * using a stream parser but others use the slightly faster string parser.
  *
+ * A block can also be provided with a single argument. That argument will be
+ * the parsed JSON document. This is useful when parsing a string that includes
+ * multiple JSON documents.
+ *
  * @param [String|IO] json JSON String or an Object that responds to read()
  * @param [Hash] options load options (same as default_options)
  */
@@ -746,6 +771,10 @@ load(int argc, VALUE *argv, VALUE self) {
  * This is a stream based parser which allows a large or huge file to be loaded
  * without pulling the whole file into memory.
  *
+ * A block can also be provided with a single argument. That argument will be
+ * the parsed JSON document. This is useful when parsing a string that includes
+ * multiple JSON documents.
+ *
  * @param [String] path path to a file containing a JSON document
  * @param [Hash] options load options (same as default_options)
  */
@@ -1211,6 +1240,14 @@ str_writer_to_s(VALUE self) {
 
 // StreamWriter
 
+/* Document-class: Oj::StreamWriter
+ * 
+ * Supports building a JSON document one element at a time. Build the IO stream
+ * document by pushing values into the document. Pushing an array or an object
+ * will create that element in the JSON document and subsequent pushes will add
+ * the elements to that array or object until a pop() is called.
+ */
+
 static void
 stream_writer_free(void *ptr) {
     StreamWriter	sw;
@@ -1251,14 +1288,10 @@ stream_writer_reset_buf(StreamWriter sw) {
     *sw->sw.out.cur = '\0';
 }
 
-/* call-seq: new(options)
- *
- * Creates a new StreamWriter.
- * @param [Hash] options formating options
- */
-/* call-seq: new(options)
+/* call-seq: new(io, options)
  *
  * Creates a new StreamWriter.
+ * @param [IO] io stream to write to
  * @param [Hash] options formating options
  */
 static VALUE
@@ -1409,7 +1442,7 @@ stream_writer_push_value(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
-/* call-seq: push_value(value, key=nil)
+/* call-seq: push_json(value, key=nil)
  *
  * Pushes a string onto the JSON document. The String must be a valid JSON
  * encoded string. No additional checking is done to verify the validity of the
@@ -1760,7 +1793,7 @@ static struct _Options	mimic_object_to_json_options = {
     No,			// class_cache
     RubyTime,		// time_format
     No,			// bigdec_as_num
-    AutoDec,		// bigdec_load
+    FloatDec,		// bigdec_load
     No,			// to_json
     Yes,		// nilnil
     Yes,		// allow_gc
@@ -1769,8 +1802,8 @@ static struct _Options	mimic_object_to_json_options = {
     10,			// create_id_len
     9,			// sec_prec
     0,			// dump_opts
-    15,			// float_prec
-    "%0.15g",		// float_fmt
+    16,			// float_prec
+    "%0.16g",		// float_fmt
 };
 
 static VALUE
@@ -1826,6 +1859,7 @@ define_mimic_json(int argc, VALUE *argv, VALUE self) {
 	mimic = rb_const_get_at(rb_cObject, rb_intern("JSON"));
     } else {
 	mimic = rb_define_module("JSON");
+	rb_define_module_function(rb_cObject, "JSON", mimic_dump_load, -1);
     }
     if (rb_const_defined_at(mimic, rb_intern("Ext"))) {
 	ext = rb_const_get_at(mimic, rb_intern("Ext"));
@@ -2018,6 +2052,7 @@ void Init_oj() {
     oj_json_create_id = rb_intern("json_create");
     oj_length_id = rb_intern("length");
     oj_new_id = rb_intern("new");
+    oj_parse_id = rb_intern("parse");
     oj_pos_id = rb_intern("pos");
     oj_read_id = rb_intern("read");
     oj_readpartial_id = rb_intern("readpartial");
@@ -2032,7 +2067,9 @@ void Init_oj() {
     oj_tv_nsec_id = rb_intern("tv_nsec");
     oj_tv_sec_id = rb_intern("tv_sec");
     oj_tv_usec_id = rb_intern("tv_usec");
+    oj_utc_id = rb_intern("utc");
     oj_utc_offset_id = rb_intern("utc_offset");
+    oj_utcq_id = rb_intern("utc?");
     oj_write_id = rb_intern("write");
 
     rb_require("oj/bag");
@@ -2048,7 +2085,6 @@ void Init_oj() {
     oj_parse_error_class = rb_const_get_at(Oj, rb_intern("ParseError"));
     oj_stringio_class = rb_const_get(rb_cObject, rb_intern("StringIO"));
     oj_struct_class = rb_const_get(rb_cObject, rb_intern("Struct"));
-    oj_time_class = rb_const_get(rb_cObject, rb_intern("Time"));
 
     allow_gc_sym = ID2SYM(rb_intern("allow_gc"));	rb_gc_register_address(&allow_gc_sym);
     ascii_only_sym = ID2SYM(rb_intern("ascii_only"));	rb_gc_register_address(&ascii_only_sym);
@@ -2079,6 +2115,7 @@ void Init_oj() {
     symbol_keys_sym = ID2SYM(rb_intern("symbol_keys"));	rb_gc_register_address(&symbol_keys_sym);
     time_format_sym = ID2SYM(rb_intern("time_format"));	rb_gc_register_address(&time_format_sym);
     unix_sym = ID2SYM(rb_intern("unix"));		rb_gc_register_address(&unix_sym);
+    unix_zone_sym = ID2SYM(rb_intern("unix_zone"));	rb_gc_register_address(&unix_zone_sym);
     use_to_json_sym = ID2SYM(rb_intern("use_to_json"));	rb_gc_register_address(&use_to_json_sym);
     xmlschema_sym = ID2SYM(rb_intern("xmlschema"));	rb_gc_register_address(&xmlschema_sym);
     xss_safe_sym = ID2SYM(rb_intern("xss_safe"));	rb_gc_register_address(&xss_safe_sym);
