@@ -47,7 +47,8 @@
 #else
 #define NUM_MAX		(FIXNUM_MAX >> 8)
 #endif
-#define EXP_MAX		1023
+//#define EXP_MAX		1023
+#define EXP_MAX		100000
 #define DEC_MAX		15
 
 static void
@@ -401,6 +402,7 @@ read_num(ParseInfo pi) {
     ni.infinity = 0;
     ni.nan = 0;
     ni.neg = 0;
+    ni.hasExp = 0;
     ni.no_big = (FloatDec == pi->options.bigdec_load);
 
     if ('-' == *pi->cur) {
@@ -436,6 +438,7 @@ read_num(ParseInfo pi) {
 		} else {
 		    zero_cnt = 0;
 		}
+		// TBD move size check here
 		ni.i = ni.i * 10 + d;
 		if (LONG_MAX <= ni.i || DEC_MAX < ni.dec_cnt - zero_cnt) {
 		    ni.big = 1;
@@ -453,6 +456,7 @@ read_num(ParseInfo pi) {
 		    zero_cnt = 0;
 		}
 		ni.dec_cnt++;
+		// TBD move size check here
 		ni.num = ni.num * 10 + d;
 		ni.div *= 10;
 		if (LONG_MAX <= ni.div || DEC_MAX < ni.dec_cnt - zero_cnt) {
@@ -463,6 +467,7 @@ read_num(ParseInfo pi) {
 	if ('e' == *pi->cur || 'E' == *pi->cur) {
 	    int	eneg = 0;
 
+	    ni.hasExp = 1;
 	    pi->cur++;
 	    if ('-' == *pi->cur) {
 		pi->cur++;
@@ -699,9 +704,9 @@ oj_num_as_value(NumInfo ni) {
 	    }
 	} else {
 	    if (ni->neg) {
-		rnum = LONG2NUM(-ni->i);
+		rnum = rb_ll2inum(-ni->i);
 	    } else {
-		rnum = LONG2NUM(ni->i);
+		rnum = rb_ll2inum(ni->i);
 	    }
 	}
     } else { // decimal
@@ -817,7 +822,7 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
 	    return oj_pi_sparse(argc, argv, pi, 0);
 	} else {
 	    rb_raise(rb_eArgError, "strict_parse() expected a String or IO Object.");
-	}
+}
     }
     if (Yes == pi->options.circular) {
 	pi->circ_array = oj_circ_array_new();
@@ -877,7 +882,6 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
     if (err_has(&pi->err)) {
 	oj_err_raise(&pi->err);
     }
-
     if (pi->options.quirks_mode == No) {
 	switch (rb_type(result)) {
             case T_NIL:
@@ -894,6 +898,5 @@ oj_pi_parse(int argc, VALUE *argv, ParseInfo pi, char *json, size_t len, int yie
 		break;
 	}
     }
-
     return result;
 }
